@@ -17,7 +17,7 @@ molpropmap = {
     "segname": "segid",
     "altloc": "altloc",
     "mass": "masses",
-    "occupancy": "beta",
+    "occupancy": "occupancy",
     "beta": "beta",
     "charge": "charge",
 }
@@ -36,6 +36,13 @@ def get_molprop(mol, molprop, analysis):
     raise RuntimeError(f"Invalid molecule property {molprop} requested")
 
 
+def _is_float(val):
+    if isinstance(val, (float, np.floating)) or (
+        isinstance(val, np.ndarray) and np.issubdtype(val.dtype, np.floating)
+    ):
+        return True
+
+
 def traverse_ast(mol, analysis, node):
     node = list(node)
     operation = node[0]
@@ -141,7 +148,9 @@ def traverse_ast(mol, analysis, node):
     if operation == "comp":
         op = node[1]
         val1, val2 = node[2], node[3]
-        if op == "=":
+        if op in ("=", "=="):
+            if _is_float(val1) or _is_float(val2):
+                return abs(val1 - val2) < 1e-6
             return val1 == val2
         if op == "<":
             return val1 < val2
@@ -317,6 +326,10 @@ class _TestAtomSelect(unittest.TestCase):
             "same fragment as within 8 of resid 100",
             "nucleic and name C3'",
             'resname C8E GR4 "200" 1PE',
+            "occupancy 0",
+            "occupancy = 0",
+            "occupancy == 0",
+            "(occupancy 1) and same beta as exwithin 3 of (occupancy 0)",
         ]
 
         pdbids = [
