@@ -763,7 +763,7 @@ class SmallMol(object):
     def view(self, *args, **kwargs):
         self.toMolecule().view(*args, **kwargs)
 
-    def getDescriptors(self, prefix="", ignore=("Ipc")):
+    def getDescriptors(self, prefix="", ignore=("Ipc",)):
         """Calculate descriptors for the molecule
 
         Returns rdkit descriptors for the molecule, like DESC_NumRotatableBonds or DESC_MolLogP.
@@ -1158,6 +1158,9 @@ class SmallMol(object):
 
         return sms
 
+    def setProp(self, key, value):
+        self._mol.SetProp(key, str(value))
+
     def __repr__(self):
         return (
             "<{}.{} object at {}>\n".format(
