@@ -4,10 +4,8 @@
 # No redistribution in whole or part
 #
 import numpy as np
-import ctypes as ct
 import os
 from moleculekit.molecule import mol_equal
-from moleculekit.support import xtc_lib
 from moleculekit.util import ensurelist, sequenceID
 import networkx as nx
 import logging
@@ -231,8 +229,9 @@ def PDBwrite(mol, filename, frames=None, writebonds=True, mode="pdb"):
 
 
 def XTCwrite(mol, filename):
-    coords = mol.coords
+    from moleculekit.xtc import write_xtc
 
+    coords = mol.coords
     nframes = mol.numFrames
 
     box = np.zeros((3, nframes), dtype=np.float32)
@@ -268,23 +267,7 @@ def XTCwrite(mol, filename):
     if not coords.flags["C_CONTIGUOUS"]:
         coords = np.ascontiguousarray(coords)
 
-    lib = xtc_lib()
-    natoms = ct.c_int(coords.shape[0])
-    nframes = ct.c_int(coords.shape[2])
-
-    cstep = step.ctypes.data_as(ct.POINTER(ct.c_int))
-    ctime = time.ctypes.data_as(ct.POINTER(ct.c_float))
-    cbox = box.ctypes.data_as(ct.POINTER(ct.c_float))
-    ccoords = coords.ctypes.data_as(ct.POINTER(ct.c_float))
-    lib["libxtc"].xtc_write(
-        ct.c_char_p(filename.encode("ascii")),
-        natoms,
-        nframes,
-        cstep,
-        ctime,
-        ccoords,
-        cbox,
-    )
+    write_xtc(filename.encode("UTF-8"), coords, box, time, step)
 
 
 def BINCOORwrite(mol, filename):
@@ -307,6 +290,7 @@ def BINCOORwrite(mol, filename):
 
 def PSFwrite(m, filename, explicitbonds=None):
     import string
+    from moleculekit.periodictable import periodictable
 
     segments = np.array(
         [segid if segid != "" else chain for segid, chain in zip(m.segid, m.chain)]
@@ -336,6 +320,10 @@ def PSFwrite(m, filename, explicitbonds=None):
         # Defaults. PSF readers will fail if any are empty since it's a space delimited format
         resname = m.resname[i] if m.resname[i] != "" else "MOL"
         atomtype = m.atomtype[i] if m.atomtype[i] != "" else "NULL"
+        # If mass is not defined take it from the element
+        mass = m.masses[i]
+        if mass == 0 and len(m.element[i]):
+            mass = periodictable[m.element[i]].mass
 
         string_format = (
             f"{m.serial[i]:>{fs['serial']}} "
@@ -345,7 +333,7 @@ def PSFwrite(m, filename, explicitbonds=None):
             f"{m.name[i]:<{fs['name']}} "
             f"{atomtype:<{fs['atomtype']}} "
             f"{m.charge[i]:>9.6f} "
-            f"{m.masses[i]:>13.4f} "
+            f"{mass:>13.4f} "
             f"{0:>11} "
         )
         print(string_format, file=f)
@@ -1122,6 +1110,28 @@ class _TestWriters(unittest.TestCase):
                 assert mol_equal(mol, mol2, exceptFields=("record"))
                 os.remove(tmpfile)
 
+    def test_psf_writer(self):
+        from moleculekit.molecule import Molecule
+        import tempfile
+
+        # This ensures the right masses are written into the psf file from the elements
+
+        reffile = os.path.join(self.testfolder, "villin.psf")
+        mol = Molecule(os.path.join(self.testfolder, "villin.pdb"))
+
+        with tempfile.TemporaryDirectory() as tmpdir:
+            tmpfile = os.path.join(tmpdir, "villin.psf")
+            mol.write(tmpfile)
+
+            with open(tmpfile, "r") as f:
+                filelines = f.readlines()
+            with open(reffile, "r") as f:
+                reflines = f.readlines()
+
+            self.assertEqual(
+                filelines, reflines, msg=f"Failed comparison of {reffile} {tmpfile}"
+            )
+
 
 if __name__ == "__main__":
     unittest.main(verbosity=2)
